@@ -138,7 +138,8 @@ extract_icons(FILE *in, char *inname, bool listmode, ExtractNameGen outfile_gen,
 				Win32RGBQuad *palette = NULL;
 				uint32_t palette_count = 0;
 				uint32_t image_size, mask_size;
-				uint32_t width, height, bit_count;
+				int32_t width, height;
+				uint32_t bit_count;
 				uint8_t *image_data = NULL, *mask_data = NULL;
 				png_structp png_ptr = NULL;
 				png_infop info_ptr = NULL;
@@ -154,16 +155,23 @@ extract_icons(FILE *in, char *inname, bool listmode, ExtractNameGen outfile_gen,
 				/* Vista icon: it's just a raw PNG */
 				if (bitmap.size == ICO_PNG_MAGIC)
 				{
+					uint32_t unsigned_width, unsigned_height;
 					fseek(in, offset, SEEK_SET);
 				
 					image_size = entries[c].dib_size;
 					image_data = xmalloc(image_size);
 					if (!xfread(image_data, image_size, in))
 						goto done;
-					
-					if (!read_png (image_data, image_size, &bit_count, &width, &height))
+
+					if (!read_png (image_data, image_size, &bit_count, &unsigned_width, &unsigned_height))
 						goto done;
-					
+
+					width = (int32_t)unsigned_width;
+					height = (int32_t)unsigned_height;
+					if ((bitmap.width > INT32_MAX/4) || (bitmap.height > INT32_MAX)) {
+						warn(_("PNG too large"));
+						goto done;
+					}
 					completed++;
 					
 					if (!filter(completed, width, height, bitmap.bit_count, palette_count, dir.type == 1,
@@ -229,11 +237,19 @@ extract_icons(FILE *in, char *inname, bool listmode, ExtractNameGen outfile_gen,
 
 					if (bitmap.clr_used != 0 || bitmap.bit_count < 24) {
 						palette_count = (bitmap.clr_used != 0 ? bitmap.clr_used : 1 << bitmap.bit_count);
+						if (palette_count > 256) {
+							warn(_("palette too large"));
+							goto done;
+						}
 						palette = xmalloc(sizeof(Win32RGBQuad) * palette_count);
 						if (!xfread(palette, sizeof(Win32RGBQuad) * palette_count, in))
 							goto done;
 						offset += sizeof(Win32RGBQuad) * palette_count;
 					}
+					if (abs(bitmap.width) > INT32_MAX/max(4, bitmap.bit_count)) {
+						warn(_("bitmap width too large"));
+						goto done;
+					}
 
 					width = bitmap.width;
 					height = abs(bitmap.height)/2;
